function gaussBayesSequentialDemo

seed = 0; randn('state', seed); rand('state', seed);

truth.mu = 0.8;
truth.sigma2 = 0.1;
Nsamp = 20;
xs = normrnd(truth.mu, sqrt(truth.sigma2), 1, Nsamp);

% batch update
xbar = mean(xs); N = Nsamp;
belPrior.mu = 0; belPrior.sigma2 = 1;
bel = updateBelief(belPrior, xbar, N, truth.sigma2);

figure(2);clf
grid = -1:0.01:3;
bel.grid = normpdf(grid, bel.mu, sqrt(bel.sigma2));
plot(grid, bel.grid, 'k-', 'linewidth', 2);
hold on
h = line([truth.mu truth.mu], [0 max(bel.grid)]);
set(h, 'color', 'r')
str = sprintf('batch update: true mean = %5.3f, post mean = %5.3f, var = %5.3f', ...
	      truth.mu, bel.mu, bel.sigma2);
title(str)
drawnow


% sequential update
colors = hsv(Nsamp+1);
symbols = ['o' 'x' 's' '>' '<' '^' 'v' ...
	   '*' 'p' 'h' '+' 'd' 'o' 'x'];

figure(1);clf
bel = belPrior;
grid = -1:0.01:3;
bel.grid = normpdf(grid, bel.mu, sqrt(bel.sigma2));
h = plot(grid, bel.grid, 'k-', 'linewidth', 2);
k = 1;
hh(k) = plotmarkers(h, symbols(k), colors(k,:), 0.2);
hold on
h = line([truth.mu truth.mu], [0 max(bel.grid)]);
set(h, 'linewidth', 2, 'color', colors(1,:));
drawnow
legendstr{k} = 'n=0';

plotndx = [1 2 10 20];
for i=1:Nsamp
  x = xs(i);
  bel = updateBelief(bel, x, 1, truth.sigma2); % single sample update
  bel.grid = normpdf(grid, bel.mu, sqrt(bel.sigma2));

  if ~ismember(i,plotndx)
    continue
  end
  k = k + 1;
  h=plot(grid, bel.grid, '-', 'linewidth', 1, 'color', colors(k,:));
  hh(k) = plotmarkers(h, symbols(k), colors(k,:), 0.1);
  h = line([x x], [0 2], 'color', colors(k,:), 'linewidth', 2);
  str = sprintf('seq update %d: true mean = %5.3f, post mean = %5.3f, var = %5.3f', ...
		i, truth.mu, bel.mu, bel.sigma2);
  title(str)
  legendstr{k} = sprintf('n=%d', i);
  drawnow
  pause
end
legend(hh, legendstr)


%%%%%%
function belPost = updateBelief(belPrior, xbar, N, sigma2);

% eqns 5.112 and 5.114 (sigma2 = observation noise level)
belPost.sigma2 = (sigma2 * belPrior.sigma2) / (N*belPrior.sigma2 + sigma2);
belPost.mu = (sigma2*belPrior.mu + N*xbar*belPrior.sigma2)/(N*belPrior.sigma2 + sigma2);

