% gaussianClassifierDemo1d

traindata = heightWeightDataLoad; 
Nclasses = length(unique(traindata.Y));
[n Ndims] = size(traindata.X);

% Fit class conditional 1D Gaussians to each feature
% params.mu(d,c) for feature d, class c
for d=1:Ndims
  for c=1:Nclasses
    ndx = find(traindata.Y == c);
    dat = traindata.X(ndx,d);
    params.mu(d,c) = mean(dat);
    params.sigma(d,c) = std(dat);
    params.n(c) = length(ndx);
  end
end

% Fit priors by counting
n = length(traindata.Y);
for c=1:Nclasses
  params.classPrior(c) = length(find(traindata.Y==c))/n;
end

% Apply model to test data
X = [72 180;
     60 100;
     68 155];
for i=1:size(X,1)
  for d=1:2
    x = X(i,d);
    pm = normpdf(x, params.mu(d,1), params.sigma(d,1)) * params.classPrior(1);
    pf = normpdf(x, params.mu(d,2), params.sigma(d,1)) * params.classPrior(2);;
    post  = pm/(pm+pf);
    fname = traindata.featureNames{d};
    fprintf('p(male|%s=%5.3f)=%5.3f\n',  fname, x,  post(1));
  end
end

% Visualize model fit
figure(2);clf
for d=1:Ndims
  dat = traindata.X(:,d);
  minx = min(dat); maxx = max(dat);
  for c=1:Nclasses
    ndx = find(traindata.Y == c);
    dat = traindata.X(ndx,d);
    subplot2(Ndims, Nclasses, d, c);
    %hist(dat);
    [counts, centers] = hist(dat);
    ncounts = normalize(counts);
    bar(centers,ncounts)
    title(sprintf('%s %s', traindata.featureNames{d}, traindata.classNames{c}));

    xs = linspace(0.5*minx, 1.5*maxx, 100);
    hold on
    ps = normpdf(xs, params.mu(d,c), params.sigma(d,c));
    ps = normalize(ps);
    sf = max(ncounts) / max(ps);
    h = plot(xs, ps * sf);
    set(h, 'linewidth', 3)
    set(gca,'xlim', [minx maxx]);
  end
end


% visualize class conditional densities
colors = {'b', 'r'}; % blue for boys, pink for girls
for d=1:Ndims
  dat = traindata.X(:,d);
  sigmaPooled = mean(params.sigma(d,:));
  %sigmaPooled = sqrt( sum(params.sigma(d,:).^2 .* params.n) / n);
  minx = min(dat); maxx = max(dat);
  figure(3+d);clf
  for c=1:Nclasses
    xs = linspace(0.5*minx, 1.5*maxx, 100);
    hold on
    ps = normpdf(xs, params.mu(d,c), params.sigma(d,c));
    ps = normalize(ps);
    h = plot(xs, ps);
    set(h, 'linewidth', 3, 'color', colors{c});
    set(gca,'xlim', [minx maxx]);
  end
  dprimed = abs((params.mu(d,2)-params.mu(d,1)))/sigmaPooled;
  title(sprintf('d''=%5.3f, %s', dprimed, traindata.featureNames{d}))
end


