function params = gaussianClassifierTrain(X,Y)
% Input:
% X is an n x d matrix
% Y is an n-vector specifying the class label (in range 1..C)
%
% Output:
% params.mu(d,c) for feature d, class c
% params.Sigma(:,:,c) covariance for class c
% params.classPrior(c)

Nclasses = length(unique(Y));
n = length(Y);
for c=1:Nclasses
  ndx = find(Y == c);
  dat = X(ndx, :);
  params.mu(:,c) = mean(dat);
  params.Sigma(:,:,c) = cov(dat);
  params.sigma(:,c) = sqrt(diag(params.Sigma(:,:,c))); % diagonal
  params.classPrior(c) = length(ndx)/n;
end
% For tied parameters
params.SigmaPooled = cov(X);
params.sigmaPooled = sqrt(diag(params.SigmaPooled));
