function p = gausspdf(X, mu, Sigma)
% Equivalent to mvnpdf in the stats toolbox
% p(i) = N(X(i,:), mu, Sigma) each ROW of x is a datacase
% and mu is d*1 or 1*d and Sigma is dxd
% 
% mvnpdf allows mu to be n*d and Sigma to be d*d*n; we do not support this

mu = mu(:)'; % ensure row vector
if length(mu)==1
  X = X(:); % ensure column vector
end
[N d] = size(X);
if length(mu) ~= d
  error('X should be N x d')
end
if ndims(Sigma) == 3
  error('gausspdf only supports dxd Sigma')
end
M = repmat(mu, N, 1); % replicate the mean across rows
denom = (2*pi)^(d/2)*sqrt(abs(det(Sigma)));
mahal = sum(((X-M)*inv(Sigma)).*(X-M),2); 
p = exp(-0.5*mahal) / (denom+eps);

