function laplaceDemoJohnson()

% Johnson and Albert p35
mode = [2; 5];          % starting guess at posterior mode
numiter = 3;          % number of iterations of Newton-Raphson

args= {};
[mode, C, logZ] = laplaceApprox(@johnsonSmokingLogpost, ...
				mode, numiter, args, @logpostGrad, @logpostHess)

% Credible interval for alpha
z = norminv(1-0.025);
sigma = sqrt(C(1,1));
credint = [mode(1) - z*sigma, mode(1) + z*sigma]

% Prob(alpha>0)
ppos = 1-normcdf(-mode(1)/sigma)


% Plot exact and approximate posterior
data.y1 = 83; data.n1 = 86; data.y2 = 72; data.n2 = 86;
[alphas, etas] = meshgrid(-1:0.1:6, 2:0.1:10);
xy = [alphas(:)'; etas(:)'];
p = exp(johnsonSmokingLogpost(xy));
p = reshape(p, size(alphas));
h = max(p(:));
vals = [0.1*h 0.01*h 0.001*h];
figure(1); clf; 
contour(alphas(1,:), etas(:,1), p, vals);
title('exact')
figure(2); clf; 
surfc(alphas, etas, p)

p2 = mvnpdf(xy', mode', C);
p2 = reshape(p2, size(alphas));
h2 = max(p2(:));
vals2 = [0.1*h2 0.01*h2 0.001*h2];
figure(3); clf; 
contour(alphas(1,:), etas(:,1), p2, vals2)
title('Laplace')
figure(4); clf; 
surfc(alphas, etas, p2)
title('Laplace')


%%%%%

function g = logpostGrad(x, args)
g = numericalGradient(@johnsonSmokingLogpost, x, args);

function H = logpostHess(x, args)
H = numericalHessian(@johnsonSmokingLogpost, x, args);

