function [w] = lassoIterRidge(X, y, t,varargin)
%Lasso using iterated ridge regression
% based on code by Mark Schmidt.

[maxIter,verbose,display,optTol,zeroThreshold,lineMinIter] = ...
   process_options(varargin,'maxIter',10000,'verbose','1','Display','none','optTol',0.0000001,'zeroThreshold',0.00001,'lineMinIter',100);
[n p] = size(X);
maxiter = maxIter;
threshold = zeroThreshold;
optTol = optTol;
lambda = t;

% Initiliaze at Ridge Regression Solution
wold = zeros(p,1);
wnew = minL2Reg(X,y,t);
zeros_new = (abs(wnew) < threshold);
zeros_old = ones(p,1);

% Start log
i = 0;
while sum(abs(wnew-wold)) > optTol && i < maxiter
    wold = wnew;
    
    % Find zero-valued elements compute update design matrix if the count
    % changed
    zeros_new = (abs(wnew) <= threshold);
    wtemp = wnew(~zeros_new);
    if sum(abs(zeros_new-zeros_old)) ~= 0
        Xtemp = X(:,abs(wnew) > threshold);
        XX = Xtemp'*Xtemp;
        Xy = Xtemp'*y;
    end
    
    % compute the new ridge regression step
    wtemp(abs(wtemp) < zeroThreshold) = Inf;
    U = chol(XX+lambda*diag(abs(1./wtemp)));
    wtemp = U \ (U'\Xy);
    
    % update the locations of zeros
     wnew = zeros(p,1);
     wnew(zeros_new==0) = wtemp;
     zeros_old = zeros_new;
end
w = wnew;
