function [w] = lassoTibs(X, y, t,varargin)
% Lasso by sequentially adding constraints,
% based on code by Mark Schmidt.

[maxIter,verbose,display,optTol] = process_options(varargin,'maxIter',10000,'verbose','1',...
  'Display','none','optTol',0.0000001);
[n p] = size(X);
warning off all;

% Start from the Least Squares Solution
w = pinv(X'*X)*(X'*y); %w = minL2(X,y);

% Compute the first constraint
E = sign(w)';
b = t*ones(size(E,1),1);

% Precompute and Initialize
XX = X'*X;
yX = -y'*X;
it = 1;
iterations = 0;

options = optimset('Display',display);
[w fval exitflag output lambda] = quadprog(XX,yX,E,b,[],[],[],[],w,options);

% while the L1-norm of the parameters is above t
while sum(abs(w)) >= t+optTol && iterations < maxIter
    % Add the new sign constraint and solve the new QP
    E = [E;sign(w)'];
    [w fval exitflag output] = quadprog(XX,yX,E,t*ones(size(E,1),1),[],[],[],[],w,options);
end
