% linRegDemoProstate
% Reproduce example on p48 of "Elements of statistical learning" 

load('prostate.mat') % from prostateDataMake
%plotmatrix([y X]) 
%figure(1);clf;pairs([y X],names([end 1:end-1]))

[n d] = size(X);
n = length(find(istrain));
XX = standardizeCols(X);

[w, stderr, Zscore, pval, confint, R2, sigma2] = ...
    lm(ytrain, XX(find(istrain),:), names(1:end-1));

% Prediction accuracy
nTest = length(find(~istrain));
Xtest = [ones(nTest,1) XX(find(~istrain),:)];
ypred = Xtest*w;
mseTest = mean((ypred - ytest).^2)
mseBase = mean((w(1) - ytest).^2)

% use stats toolbox
X = [ones(length(ytrain),1) XX(find(istrain),:)];
[b, bint, r, rint, stats] = regress(ytrain, X);
% b(j) is coefficient j, bint(j,:) = lower and upper 95% conf interval
% r(i) = residual for case i, rint(i,:)  = lower and upper 95% conf interval
% stats = [Rsquared, Fstat, pval for Fstat, error variance]
assert(approxeq(b, w))
assert(approxeq(bint, confint))
assert(approxeq(stats(1), R2))
assert(approxeq(stats(4), sigma2))


% F test for dropping all coefficients
[F, pvalF] = Ftest(ytrain, XX(find(istrain),:), [], 1:d);
assert(approxeq(stats(2), F))
assert(approxeq(stats(3), pvalF))

% F test for dropping all "non significant" coefficients
ZZ = Zscore(2:end); % remove offset
nonsig = find(abs(ZZ)<2);
sig = find(abs(ZZ)>2);
[F, pvalF] = Ftest(ytrain, XX(find(istrain),:), sig, 1:d);
assert(approxeq(F, 1.67)) % Elements eqn 3.16
assert(approxeq(pvalF, 0.17))

% Prove Elements exercise 3.1 by example
for i=1:d
  [FF(i), pvalF] = Ftest(ytrain, XX(find(istrain),:), setdiff(1:d,i), 1:d);
end
assert(approxeq(FF, Zscore(2:end).^2))
