function [w, stderr, Zscore, pval, confint, R2, sigma2] = lm(y, XX, names);
% simulate the effect of R's lm function
% Each row ox X is a training case, excluding the 1 term

if nargin < 3, names = []; end

[n d] = size(XX);
X = [ones(n,1) XX];
w = X\y; % mle
yhat = X*w;
rss = sum((y - yhat).^2);
R2 = 1-rss/sum((y-mean(y)).^2);
sigma2 = rss/(n-d-1); % unbiased estimate
Sigma = pinv(X'*X)*sigma2; % covariance matrix of w
stderr = sqrt(diag(Sigma));
Zscore = w ./ stderr;
dof = n-d-1;
pval = 1-tcdf(abs(Zscore),dof) + tcdf(-abs(Zscore),dof);
alpha = 0.95;
tc = tinv(1-(1-alpha)/2, dof); 
confint = [w-tc*stderr w+tc*stderr]
if isempty(names), return; end

names2 = {'intercept', names{:}};
fprintf('linear regression n=%d, d=%d, R-squared=%10.5f\n', n, d, R2);
fprintf('%10s %10s %10s %10s %10s %10s %10s\n', ...
	'name', 'estimate', 'std err', 't value', 'p(>|t|)', 'lower', 'upper');
for i=1:d+1
  fprintf('%10s %10.5f %10.5f %10.5f %10.5f %10.5f %10.5f\n', ...
	  names2{i}, w(i), stderr(i), Zscore(i),  pval(i), confint(i,1), confint(i,2));
end
