function lsaDemoSmall()
% Based on a tutorial by Genevieve Gorrell
% http://www.dcs.shef.ac.uk/~genevieve/lsa_tutorial.htm

sentence{1} = 'the man walked the dog.';
sentence{2} = 'the man took the dog to the park';
sentence{3} = 'the dog went to the park';
N = length(sentence);
words = [];
for i=1:N
  w = regexp(sentence{i}, '(\w+)', 'match');
  words = [words w];
end
words = unique(words);

% to facilitate comparison with the tutorial, we sort the words as follows
words = {'the', 'man', 'walked', 'dog', 'took', 'to', 'park', 'went'};

% Now build term-document matrix
W =  length(words);
M = zeros(W, N);
for i=1:N
  M(:,i) = M(:,i) + sentenceToWordCount(sentence{i}, words);
end

% truncated SVD
K = 2;
[U, S, V] = svds(M, 2);

% add test data to word-document matrix
query = 'the dog walked';
x = sentenceToWordCount(query, words);
%x = x/norm(x);
M(:,N+1) = x;

% embed all data into low dim space and plot
%X = U'*M;
ndoc = size(M,2);
X = zeros(K, ndoc);
for i=1:ndoc
  %X(:,i) = U'*(M(:,i) / norm(M(:,i)));
  X(:,i) = U'*M(:,i);
  X(:,i) = X(:,i) / norm(X(:,i));
  %X(:,i) = U'*(normalize(M(:,i)));
end

figure(1);clf;
for i=1:ndoc
  plot(X(1,i), X(2,i), 'o');
  hold on
  h=text(X(1,i), X(2,i), sprintf('%d', i)); set(h,'fontsize',15);
end

% compute pairwise similarities  using dot products in low dim space
simMat = X'*X;
simMat = setdiag(simMat, 0);
[junk, bestMatch] = max(simMat(4,:));
fprintf('best match to query is document %d\n', bestMatch);


%%%%%%


function v = sentenceToWordCount(s, words)

tokens = regexp(s, '(\w+)', 'match');
v = zeros(length(words),1);
for j=1:length(tokens)
  k = strmatch(tokens{j}, words, 'exact');
  v(k) = v(k) + 1;
end

