function [beta] = probitregSampleParams(X,y,v,numSamples)
% Sample parameters from posterior for probit regression.
% Uses method of Holmes and Held.
% X(i,:) = case i
% y(i) = 0/1
% Prior on weights is N(0,Sigma)
% beta(:,s) = sample s
%
% Based on code written by Mark Schmidt

[n,p] = size(X);
V = pinv(X'*X + pinv(Sigma));
L = chol(V)'; % Lower triangular Cholesky
S = V*X';

for i= 1:n
    H(i) = X(i,:)*S(:,i); % diagonal elements of hat matrix
    W(i) = H(i)/(1-H(i));
    Q(i) = W(i)+1;
end


Z = abs(randn(n,1)).*sign(y-.5); % initial guess
mu = S*Z; % Conditional means of beta

for s = 1:numSamples
  for i = 1:n
    z_old = Z(i);
    m = X(i,:)*mu;
    m = m - W(i)*(Z(i)-m);
    Z(i) = sign(y(i)-.5)*truncatedNormalSample(sign(y(i)-.5)*m,Q(i),0);
    mu = mu + (Z(i)-z_old)*S(:,i);
  end
  T = mvnrnd(zeros(p,1),eye(p))';
  beta(:,s)=mu + L*T;
end

