function rainfallBootstrapDemo()

X = dlmread('rainfallData.txt');
X = X'; X = X(:); % concatenate across rows, not columns
X = X(1:end-5); % removing trailing 0s

B  = 1000;
[asampMOM, bsampMOM] = bootstrap(B, X, @mom);
[asampMLE, bsampMLE] = bootstrap(B, X, @mle);

figure;
subplot(2,2,1)
hist(asampMOM);
title(sprintf('a MOM, se = %5.3f', std(asampMOM)));
set(gca,'xlim',[0.1 0.7])

subplot(2,2,2)
hist(bsampMOM);
title(sprintf('b MOM, se = %5.3f', std(bsampMOM)))
set(gca,'xlim',[0.5 3.5])

subplot(2,2,3)
hist(asampMLE); 
title(sprintf('a MLE, se = %5.3f', std(asampMLE)))
set(gca,'xlim',[0.1 0.7])

subplot(2,2,4)
hist(bsampMLE);
title(sprintf('b MLE, se = %5.3f', std(bsampMLE)))
set(gca,'xlim',[0.5 3.5])


%%%%%%
function [asamp, bsamp] = bootstrap(B, X, estimator)

N = length(X);
[agen, bgen] = feval(estimator, X);
Xsamp = gamrnd(agen, 1/bgen, B, N);
for b=1:B
  [asamp(b), bsamp(b)] = feval(estimator, Xsamp(b,:));
end

%%%%%%

function [a,b] = mom(X)
% method of moments estimate
xbar = mean(X);
s2hat = var(X);
a = xbar^2/s2hat;
b  = xbar/s2hat;

%%%%%%

function [a,b] = mle(X)
% max likelihood estimate
params = gamfit(X);
a = params(1);
b = 1/params(2);
