% Demo of fitting a Gamma distribution to the rainfall data used in Rice (1995) p383
% The data records the average amount of rain (in inches) in southern Illinois
% during each storm over the years 1960 - 1964
X = dlmread('rainfallData.txt');
X = X'; X = X(:); % concatenate across rows, not columns
X = X(1:end-5); % removing trailing 0s

% method of moments estimate
xbar = mean(X)
s2hat = var(X)
a = xbar^2/s2hat
b  = xbar/s2hat

% MLE
[params, ci] = gamfit(X)
aMLE = params(1)
bMLE = 1/params(2)

[v, binc] = hist(X);
h = binc(2)-binc(1);
N = length(X);
areaH = h*N;
figure(1);clf;bar(binc, v);
vv = v/areaH;
assert(approxeq(1, sum(vv*h))) % vv is a valid pmf
figure(2);clf;bar(binc, vv) 

if 1
%xs = linspace(min(X), max(X), 100);
%xs = linspace(binc(1), binc(end), 100);
xs = linspace(0.05,  binc(end), 100);
dx = xs(2)-xs(1);
p = gampdf(xs, a, 1/b);
areaP = sum(p*dx);
% rescale pdf so area under its curve equals area under histogram
%figure(3); clf
figure(1); hold on
%plot(xs, (p/areaP)*areaH, 'r-', 'linewidth', 3)
plot(xs, p*areaH, 'r-', 'linewidth', 3)
%figure(4); clf
figure(2); hold on
plot(xs, p, 'r-', 'linewidth', 3)
%plot(xs, p/areaP, 'r-', 'linewidth', 3)
end

if 0
xs = binc;
dx = xs(2)-xs(1);
p = gampdf(xs, a, 1/b);
areaP = sum(p*dx);
% rescale pdf so area under its curve equals area under histogram
figure(5); hold on
%figure(1); hold on
%plot(xs, (p/areaP)*areaH, 'r-', 'linewidth', 3)
plot(xs, p*areaH, 'ro-', 'linewidth', 3)
figure(6); hold on
%figure(2); hold on
plot(xs, p, 'ro-', 'linewidth', 3)
end
